#!/bin/bash
set -e
echo "USAGE: ./deploy.sh VERSION"

operator_image="${operator_image:-quay.io/domino/domsed}"
platform_namespace="${platform_namespace:-domino-platform}"
compute_namespace="${compute_namespace:-domino-compute}"
name="operator"

VERSION=$1

if [ -z "$VERSION" ]
then
      echo "Please specify a version."
      exit 1
fi

secret="${name}-webhook-certs"
service="${name}-webhook-svc"

if [ ! -x "$(command -v openssl)" ]; then
    echo "openssl not found"
    exit 1
fi


#
# CRD
#

function kube_minor_version() {
    MINOR_VERSION=$(kubectl version -ojson | jq -rc .serverVersion.minor | sed 's/[^0-9]//g')
}

kube_minor_version

echo "Found Kubernetes Minor Version: $MINOR_VERSION"

if [ $MINOR_VERSION -ge 16 ]; then
    echo "Stable CRD API"
    kubectl apply -f manifests/apps.dominodatalab.com_mutations_crd_v16+.yaml  
elif [ $MINOR_VERSION -ge 15 ]; then
    echo "Beta API; Prunning Supported"
    kubectl apply -f manifests/apps.dominodatalab.com_mutations_crd_v15.yaml  
else
    echo "Beta API; Prunning Unsupported"
    kubectl apply -f manifests/apps.dominodatalab.com_mutations_crd_v14-.yaml  
fi

csrName=${service}.${platform_namespace}
tmpdir=$(mktemp -d)
echo "creating certs in tmpdir ${tmpdir} "

cat <<EOF >> ${tmpdir}/req.conf
[req]
req_extensions = v3_req
distinguished_name = req_distinguished_name
[req_distinguished_name]
[ v3_req ]
basicConstraints = CA:FALSE
keyUsage = nonRepudiation, digitalSignature, keyEncipherment
extendedKeyUsage = serverAuth
subjectAltName = @alt_names
[alt_names]
DNS.1 = ${service}
DNS.2 = ${service}.${platform_namespace}
DNS.3 = ${service}.${platform_namespace}.svc
EOF


# Create CA Key
openssl genrsa -out ${tmpdir}/ca.key 2048

# Create CA Cert
openssl req -new -key ${tmpdir}/ca.key -x509 -out ${tmpdir}/ca.crt -days 3650 -subj "/CN=ca"


# Create Server Key and Signing Request
openssl req -new -nodes -newkey rsa:2048 -keyout ${tmpdir}/server.key -out ${tmpdir}/server.req -batch -config ${tmpdir}/req.conf -subj "/"

# Create Signed Server Cert
openssl x509 -req -in ${tmpdir}/server.req -CA ${tmpdir}/ca.crt -CAkey ${tmpdir}/ca.key -CAcreateserial -out ${tmpdir}/server.crt -days 3650 -sha256 -extensions v3_req -extfile ${tmpdir}/req.conf


# create the secret with CA cert and server cert/key
kubectl create secret generic ${secret} \
        --from-file=key.pem=${tmpdir}/server.key \
        --from-file=cert.pem=${tmpdir}/server.crt \
        --dry-run -o yaml |
    kubectl -n ${platform_namespace} apply -f -

CA_BUNDLE=$(cat ${tmpdir}/server.crt | base64 | tr -d '\n\r')

echo "Creating Service Account"
cat <<EOF | kubectl create -n ${platform_namespace} -f -
apiVersion: v1
kind: ServiceAccount
metadata:
  name: ${name}-webhook
---
kind: Role
apiVersion: rbac.authorization.k8s.io/v1
metadata:
  name: ${name}-webhook
rules:
- apiGroups:
  - apps.dominodatalab.com
  resources:
  - "mutations"
  verbs:
  - "*"
---
kind: RoleBinding
apiVersion: rbac.authorization.k8s.io/v1
metadata:
  name: ${name}-webhook
subjects:
- kind: ServiceAccount
  name: ${name}-webhook
  namespace: ${platform_namespace}
roleRef:
  kind: Role
  name: ${name}-webhook
  apiGroup: rbac.authorization.k8s.io
EOF

cat <<EOF | kubectl create -n ${compute_namespace} -f -
kind: Role
apiVersion: rbac.authorization.k8s.io/v1
metadata:
  name: ${name}-webhook
rules:
- apiGroups:
  - ""
  resources:
  - "serviceaccounts"
  verbs:
  - "get"
  - "create"
- apiGroups:
  - ""
  resources:
  - "services"
  - "pods"
  verbs:
  - "get"
  - "watch"
  - "list"
- apiGroups:
  - "rbac.authorization.k8s.io" 
  resources:
  - "rolebindings"
  verbs:
  - "patch"
- apiGroups:
  - "" 
  resources:
  - "secrets"
  verbs:
  - "create"
  - "get"
  - "update"
  - "patch"
- apiGroups:
  - "policy" 
  resources:
  - "podsecuritypolicies"
  resourceNames:
  - "domino-${platform_namespace}-nucleus-cg2"
  verbs:
  - "use"
---
kind: RoleBinding
apiVersion: rbac.authorization.k8s.io/v1
metadata:
  name: ${name}-webhook
subjects:
- kind: ServiceAccount
  name: ${name}-webhook
  namespace: ${platform_namespace}
roleRef:
  kind: Role
  name: ${name}-webhook
  apiGroup: rbac.authorization.k8s.io
EOF

echo "Creating Deployment"
cat <<EOF | kubectl create -n ${platform_namespace} -f -
apiVersion: apps/v1
kind: Deployment
metadata:
  name: ${name}-webhook
  labels:
    app: ${name}-webhook
spec:
  replicas: 1
  selector:
    matchLabels:
      app: ${name}-webhook
  template:
    metadata:
      labels:
        app: ${name}-webhook
        mongodb-replicaset-client: "true"
    spec:
      serviceAccountName: ${name}-webhook
      automountServiceAccountToken: true
      nodeSelector:
        dominodatalab.com/node-pool: platform
      containers:
      - name: domsed
        securityContext:
          runAsUser: 1000
          runAsGroup: 1000
          allowPrivilegeEscalation: false
          capabilities:
            drop:
              - all
        image: ${operator_image}:${VERSION}
        ports:
        - containerPort: 5000
        livenessProbe:
          httpGet:
            path: /healthz
            port: 5000
            scheme: HTTPS
          initialDelaySeconds: 20
          failureThreshold: 2
          timeoutSeconds: 5
        readinessProbe:
          httpGet:
            path: /healthz
            port: 5000
            scheme: HTTPS
          initialDelaySeconds: 20
          failureThreshold: 2
          timeoutSeconds: 5
        imagePullPolicy: Always
        env:
        - name: PLATFORM_NAMESPACE
          value: ${platform_namespace}
        - name: MONGO_PASSWORD
          valueFrom:
            secretKeyRef:
              key: password
              name: mongodb-replicaset-admin
        volumeMounts:
          - name: certs
            mountPath: /ssl
            readOnly: true
      volumes:
        - name: certs
          secret:
            secretName: ${name}-webhook-certs
EOF

echo "Creating Service"
cat <<EOF | kubectl create -n ${platform_namespace} -f -
apiVersion: v1
kind: Service
metadata:
  labels:
    app: ${name}-webhook
  name: ${service}
  namespace: ${platform_namespace}
spec:
  ports:
  - name: https
    port: 443
    targetPort: 5000
  selector:
    app: ${name}-webhook
  sessionAffinity: None
  type: ClusterIP
EOF

if kubectl get peerauthentications ; then
echo "Creating PeerAuthentication"
cat <<EOF | kubectl apply -f -
apiVersion: security.istio.io/v1beta1
kind: PeerAuthentication
metadata:
  name: ${name}-webhook
  namespace: ${platform_namespace}
spec:
  selector:
    matchLabels:
      app: ${name}-webhook
  mtls:
    mode: DISABLE
EOF
fi

# Wait for the app to actually be up before starting the webhook.
let tries=1
availreps=""
while [[ ${tries} -lt 10 && "${availreps}" != "1" ]]; do
  echo "Checking deployment, try $tries"
  kubectl get deployment -n ${platform_namespace} ${name}-webhook
  availreps=$(kubectl get deployment -n ${platform_namespace} ${name}-webhook -o jsonpath='{.status.availableReplicas}')
  let tries+=1
  sleep 10
done

if [[ ${availreps} != "1" ]]; then
  echo "Deployment never became available, exiting."
  exit 1
fi

cat <<EOF | kubectl create -n ${platform_namespace} -f -
apiVersion: networking.k8s.io/v1
kind: NetworkPolicy
metadata:
  name: ${name}-webhook
spec:
  ingress:
  - from:
    - ipBlock:
        cidr: 0.0.0.0/0
    ports:
    - port: 5000
      protocol: TCP
  podSelector:
    matchLabels:
      app: operator-webhook
  policyTypes:
  - Ingress
EOF

cat <<EOF | kubectl create -n ${platform_namespace} -f -
apiVersion: admissionregistration.k8s.io/v1
kind: MutatingWebhookConfiguration
metadata:
  name: ${name}-webhook
webhooks:
  - name: ${name}-webhook.k8s.twr.io
    clientConfig:
      service:
        name: ${service}
        namespace: ${platform_namespace}
        path: "/"
      caBundle: ${CA_BUNDLE}
    namespaceSelector:
      matchExpressions:
      - key: ${name}-enabled
        operator: Exists
    rules:
      - operations:
          - "CREATE"
        apiGroups:
          - "*"
        apiVersions:
          - "*"
        resources:
          - "pods"
          - "sparkclusters"
          - "ingresses"
        scope: "Namespaced"
    failurePolicy: Ignore
EOF

kubectl label namespace ${compute_namespace} ${name}-enabled=true
